# ControllerLoader
#
# Adds controller loading functionality to a Shot app. Will attempt to load controllers in the
# 'app/controllers' folder in the local working directory. If the file is not found, or an appropriate
# filesystem is not set up, a ControllerLoadException will be raised.
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/

require 'shot'
require_relative './controller_load_exception'

class String
	# Converts a CamelCased string to a underscore_string
	def underscore
		self.gsub(/::/, '/').
				gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
				gsub(/([a-z\d])([A-Z])/,'\1_\2').
				tr("-", "_").
				downcase
	end

	def camelize
		parts = self.split '_'
		parts.map! do |part| part.capitalize end
		parts.join ''
	end
end

class ControllerLoader < Loader
	def initialize(client)
		super(client)
		@type = 'controller'
	end

	def get(controller)
		if controller_exists? controller
			if direct_path_to_controller? controller
				load controller

				controller_class_name = get_controller_class_from_path controller
				controller_object = get_controller_object controller_class_name
				controller_instance = controller_object.new @client

				call_setup_method controller_object, controller_instance

				controller_instance

			else
				load "app/controllers/#{controller.underscore}_controller.rb"

				controller_class_name = get_controller_class controller
				controller_object = get_controller_object controller_class_name
				controller_instance = controller_object.new @client

				call_setup_method controller_object, controller_instance

				controller_instance
			end
		else
			raise ControllerLoadException.new "Error loading controller #{controller}. Please verify that it exists at app/controllers/#{controller.underscore}.rb"
		end
	end

	private

	def controller_exists?(controller)
		File.exists? controller or File.exists? "app/controllers/#{controller.underscore}_controller.rb"
	end

	def direct_path_to_controller?(controller)
		File.exists? controller
	end

	def call_setup_method(controller_object, controller_instance)
		controller_instance.send 'setup' unless not controller_object.method_defined? :setup
	end

	def get_controller_object(class_name)
		Object.const_get class_name
	end

	def get_controller_class(controller)
		"#{controller}Controller"
	end

	def get_controller_class_from_path(controller)
		last_part = controller.split('/')[-1]
		last_part['.rb'] = ''
		last_part.camelize
	end
end